import { useRef, useEffect } from 'react';
import { select, map, range } from 'd3';

// D3 as React Component
export const D3Component = ({
  d3Data,
  innerWidth,
  innerHeight
}) => {
  // useRef to access the raw DOM -> useEffect to grab the DOM after it is created
  const gRef = useRef();
  useEffect(() => {
    // Do D3 stuffs
    const data = d3Data.filter(d => d.value !== null);
    const g = select(gRef.current);
    //console.log(data)
    
    const V = d3.map(data, d => d.value);
    const G = d3.map(data, d => d.category);
    console.log(V)
    const I = d3.range(V.length).filter(i => V[i] > 0);
		
    let groups = I.map(i => G[i]);
    groups = G && new d3.InternSet(groups);
    
    // Compute Labels and Titles
    const L = d3.map(data, d => d.id.split('|')); // circleText
    console.log(L)
    const T = d3.map(data, d => `${d.id}\n${d.category}`) // tooltip
    
    // Color Scale
    const colors = d3.schemeTableau10;
    const colorScale = d3.scaleOrdinal().domain(groups).range(colors);
    const colorMaps = d3.map(data, d => colorScale(d.category));
    
    // Compute Layout
    const root = d3.hierarchy({name:"root", children: I})
    	.sum(i => +V[i]);
    
    const pack = d3.pack()
    	.size([innerWidth, innerHeight])
    	.padding(220);
    
    const packOut = pack(root);
    console.log(L)
    const leaf = g.selectAll("g").data(packOut.leaves())
    	.join("g")
    		.attr("class", "categoryCircle")
    		.attr("transform", d => `translate(${d.x}, ${d.y})`)
    		.style('font-size', d => {
      console.log(Math.log2(d.r)*3)
      return 10});
    
    leaf.selectAll("circle").data(d => d)
    	.join('circle')
    	.attr("stroke", 'black')
    	.attr('stroke-width', 0)
      .attr('fill', (d,i) => colorMaps[d.data])
    	.attr('r', d => Math.log2(d.r)*10);
    
    leaf.selectAll('text').data(d => d)
    	.join('text')
    		.selectAll('tspan').data(d => L[d.data])
        .join('tspan')
        .text(d => d)
    		.attr('x', 0)
    		.attr("y", (d,i) => i*15)
        .attr("fill-opacity", 0.9)
    		.attr('text-anchor', 'middle');
    
    leaf.append('title')
    	.text(d => T[d.data]);
    
    const BubbleChart = () => {};
    
	
    const colorLegend = (selection, props) => {
      const { 
        colorScale, 
        spacing, 
        textOffset
      } = props;

      const groups = selection.selectAll('g')
        .data(colorScale.domain());
      const groupsEnter = groups
        .enter().append('g')
          .attr('class', 'tick');
      groupsEnter
        .merge(groups)
          .attr('transform', (d, i) => 
            `translate(0, ${i * spacing})`
          );
      groups.exit().remove();

      groupsEnter.append('rect')
        .merge(groups.select('rect'))
          .attr('width', 30)
          .attr('height', 20)
          .attr('fill', colorScale);  // equivalent to d => colorScale(d)
      // add text labels
      groupsEnter.append('text')
      .merge(groups.select('text'))
        .text(d => d)
        .attr('x', textOffset)
        .attr('dy', '0.89em');  // magic value for aligning text vertically to the middle of DOMs
    }
    
    const legendG = g.append('g').attr('class', 'legend')
    	.attr('transform', `translate(${700}, ${30})`);
    legendG.call(colorLegend, {
      colorScale,
      spacing: 30,
      textOffset: 40
    })
    
  }, [d3Data]);
  
  return <g ref={gRef} />;
};
