import { range } from 'd3';
import React, { useState, useMemo, useCallback } from 'react';
import ReactDOM from 'react-dom';
import { VizContainer } from './VizContainer';
import { useData } from './useData';
import { D3Component } from './D3Component';

const width = 960;
const height = 475;
const margin = { top: 10, right: 10, bottom: 10, left: 10 };

const App = () => {
  // States and Hooks before data load
	const [d3Data, setD3Data] = useState(range(5));
  const [circleSize, setCircleSize] = useState(10);
  
  // useMemo Example
  const [innerWidth, innerHeight] = useMemo(()=>{
    const a = width - margin.right - margin.left;
    const b = height - margin.top - margin.bottom;
    return [a, b]
  }, [width, height, margin]);
  
  // useCallback Example (define callback functions in useCallback hook)
  const onDataCountChange = useCallback((e) => {
    setD3Data(range(e.target.value));
  }, []);

  // Load Data
  const data = useData();
  if (!data) {
    return <pre>Loading...</pre>;
  };
  // console.log("d3Data", d3Data)
  // console.log("circleSize", circleSize)
  
  return (
    <>
      <div>
        <label for="data-count">Circle Data Count: </label>
        <input type="number" id="data-count" onChange={onDataCountChange}/>
        <label for="circle-size">Circle Size: </label>
        <input type="number" id="circle-size" onChange={ (e) => {
            setCircleSize(e.target.value);
        }}/>
      </div>
      <VizContainer
        id="viz-container"
        width={width}
        height={height}
        margin={margin}
        innerWidth={innerWidth}
        innerHeight={innerHeight}
      >
        <D3Component d3Data={data} innerWidth={innerWidth} innerHeight={innerHeight} />
      </VizContainer>
    </>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
